@@ -5,22 +5,23 @@ use warnings;
 
 # Use module and create needed exceptions
 use Exception::Base
-    ':all',
     'verbosity' => 3,
     'Exception::IO',
     'Exception::FileNotFound' => { isa => 'Exception::IO', has => 'filename' };
 
 sub func1 {
     # try / catch
-    try 'Exception::Base' => eval {
+    eval {
         my $file = '/notfound';
         open my $fh, $file
-            or throw 'Exception::FileNotFound' =>
-                     message=>'Can not open file', filename=>$file;
+            or Exception::FileNotFound->throw(
+                   message=>'Can not open file', filename=>$file
+               );
 	close $fh;
     };
 
-    if (catch 'Exception::IO' => my $e) {
+    if ($@) {
+        my $e = Exception::Base->catch;
         # $e is an exception object for sure, no need to check if is blessed
         warn "Exception caught";
         if ($e->isa('Exception::FileNotFound')) {
